global feats_train;
global feats_test;

global distance;
global kernel;
global pie;

global kernel_name;
global kernel_feature_class;
global kernel_feature_type;
global kernel_data;
global kernel_data_train;
global kernel_data_test;
global kernal_matrix_train;
global kernel_matrix_test;
global kernel_alphabet;
global kernel_order;
global kernel_gap;
global kernel_reverse;
global kernel_normalizer;
global kernel_arg1_gamma;
global kernel_arg0_c;
global kernel_arg2_coef0;
global kernel_arg1_diag;
global kernel_arg0_degree;
global kernel_arg1_degree;
global kernel_arg2_distance;
global kernel_arg1_inhomogene;
global kernel_arg2_inhomogene;
global kernel_arg2_inner_degree;
global kernel_arg1_k;
global kernel_arg1_length;
global kernel_arg2_max_shift;
global kernel_arg1_normalization;
global kernel_arg3_outer_degree;
global kernel_arg0_scale;
global kernel_arg3_shift_step;
global kernel_arg0_size;
global kernel_arg1_size;
global kernel_arg1_use_sign;
global kernel_arg0_width;
global kernel_arg1_width;
global kernel_arg2_width;

global kernel_subkernel0_name;
global kernel_subkernel0_accuracy;
global kernel_subkernel0_data_train;
global kernel_subkernel0_data_test;
global kernel_subkernel0_data_class;
global kernel_subkernel0_feature_class;
global kernel_subkernel0_feature_type;
global kernel_subkernel0_arg0_size;
global kernel_subkernel0_arg1_degree;
global kernel_subkernel0_arg1_width;
global kernel_subkernel0_alphabet;
global kernel_subkernel0_seqlen;
global kernel_subkernel1_name;
global kernel_subkernel1_accuracy;
global kernel_subkernel1_data_train;
global kernel_subkernel1_data_test;
global kernel_subkernel1_data_class;
global kernel_subkernel1_feature_type;
global kernel_subkernel1_feature_class;
global kernel_subkernel1_arg0_size;
global kernel_subkernel1_arg1_degree;
global kernel_subkernel1_arg2_inhomogene;
global kernel_subkernel1_alphabet;
global kernel_subkernel1_seqlen;
global kernel_subkernel2_name;
global kernel_subkernel2_accuracy;
global kernel_subkernel2_data_train;
global kernel_subkernel2_data_test;
global kernel_subkernel2_data_class;
global kernel_subkernel2_feature_type;
global kernel_subkernel2_feature_class;
global kernel_subkernel2_arg0_size;
global kernel_subkernel2_alphabet;
global kernel_subkernel2_seqlen;
global kernel_symdata;

global distance_name;
global distance_feature_class;
global distance_feature_type;
global distance_data_train;
global distance_data_test;
global distance_matrix_train;
global distance_matrix_test;
global distance_alphabet;
global distance_order;
global distance_gap;
global distance_reverse;
global distance_arg0_use_sign;
global distance_arg0_k;

global classifier_name;
global classifier_feature_class;
global classifier_feature_type;
global classifier_data_train;
global classifier_data_test;
global classifier_type;
global classifier_labels;
global classifier_label_type;
global classifier_bias;
global classifier_bias_enabled;
global classifier_epsilon;
global classifier_tube_epsilon;
global classifier_max_train_time;
global classifier_linadd_enabled;
global classifier_batch_enabled;
global classifier_num_threads;
global classifier_batch_enabled;
global classifier_k;
global classifier_gamma;
global classifier_C;
global classifier_classified;
global classifier_accuracy;
global classifier_alpha_sum;
global classifier_sv_sum;
global classifier_alphabet;

global regression_name;
global regression_num_threads;
global regression_labels;
global regression_type;
global regression_C;
global regression_epsilon;
global regression_tube_epsilon;
global regression_tau;
global regression_bias;
global regression_alpha_sum;
global regression_sv_sum;
global regression_classified;
global regression_accuracy;

global clustering_name;
global clustering_max_iter;
global clustering_k;
global clustering_merges;
global clustering_radi;
global clustering_merge_distance;
global clustering_accuracy;

global distribution_name;
global distribution_feature_class;
global distribution_feature_type;
global distribution_data_train;
global distribution_data_test;
global distribution_alphabet;
global distribution_order;
global distribution_gap;
global distribution_reverse;
global distribution_N;
global distribution_M;
global distribution_pseudo;
global distribution_accuracy;
global distribution_likelihood;

global topfk_name;
global topfk_N;
global topfk_M;
global topfk_accuracy;
global topfk_likelihood;
global topfk_feature_class;
global topfk_feature_type;
global topfk_data_train;
global topfk_data_test;
global topfk_alphabet;
global topfk_order;
global topfk_gap;
global topfk_reverse;
global topfk_pseudo;

global pos_train;
global pos_test;
global neg_train;
global neg_test;

global preproc_name;
