package shogun;

message VectorHeader
{
	required uint64 len=1;
	required uint32 num_messages=2;
}

message MatrixHeader
{
	required uint64 num_cols=1;
	required uint64 num_rows=2;
	required uint32 num_messages=3;
}

message SparseMatrixHeader
{
	required uint64 num_vectors=1;
	required uint64 num_features=2;
	repeated uint64 num_feat_entries=3;
}

message StringListHeader
{
	required uint64 num_str=1;
	required uint64 max_string_len=2;
	repeated uint64 str_len=3;
}