/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2012 Fernando José Iglesias García
 * Copyright (C) 2012 Fernando José Iglesias García
 */

#ifndef __STATE_MODEL_TYPES_H__
#define __STATE_MODEL_TYPES_H__

namespace shogun
{

/** state model type */
enum EStateModelType
{
	SMT_UNKNOWN = 0,
	SMT_TWO_STATE = 1,
};

} /* namespace shogun */

#endif /* __STATE_MOTEL_TYPES_H__ */
