/* This software is distributed under BSD 3-clause license (see LICENSE file).
 *
 * Copyright (c) 2012-2013 Sergey Lisitsyn
 */

#ifndef TAPKEE_DEFINES_STDTYPES_H_
#define TAPKEE_DEFINES_STDTYPES_H_

#ifndef TAPKEE_INTERNAL_VECTOR
	#include <vector>
	#define TAPKEE_INTERNAL_VECTOR std::vector
#endif
#ifndef TAPKEE_INTERNAL_PAIR
	#include <utility>
	#define TAPKEE_INTERNAL_PAIR std::pair
#endif
#ifndef TAPKEE_INTERNAL_MAP
	#include <map>
	#define TAPKEE_INTERNAL_MAP std::map
#endif

#endif
