/* This software is distributed under BSD 3-clause license (see LICENSE file).
 *
 * Copyright (c) 2012-2013 Sergey Lisitsyn
 */

#ifndef TAPKEE_MAIN_H_
#define TAPKEE_MAIN_H_

/* Tapkee includes */
#include <shogun/lib/tapkee/embed.hpp>
#include <shogun/lib/tapkee/chain_interface.hpp>
/* End of Tapkee includes */

#endif
